@@ -71,6 +71,20 @@ KyberMode::Mode kyber_mode_from_string(std::string_view str) {
    throw Invalid_Argument(fmt("'{}' is not a valid Kyber mode name", str));
 }
 
+/**
+ * Constant time implementation for computing an unsigned integer division
+ * with KyberConstants::Q = 3329.
+ *
+ * It enforces the optimization of various compilers,
+ * replacing the division operation with multiplication and shifts.
+ *
+ * @returns (a / KyberConstants::Q)
+ */
+uint16_t ct_int_div_kyber_q(uint32_t a) {
+   const uint64_t tmp = (static_cast<uint64_t>(a) * 989558401UL) >> 32;
+   return static_cast<uint16_t>((tmp + ((a - tmp) >> 1)) >> 11);
+}
+
 }  // namespace
 
 KyberMode::KyberMode(Mode mode) : m_mode(mode) {}
@@ -102,6 +116,30 @@ std::string KyberMode::to_string() const {
    BOTAN_ASSERT_UNREACHABLE();
 }
 
+bool KyberMode::is_90s() const {
+   return m_mode == Kyber512_90s || m_mode == Kyber768_90s || m_mode == Kyber1024_90s;
+}
+
+bool KyberMode::is_modern() const {
+   return !is_90s();
+}
+
+bool KyberMode::is_available() const {
+#if defined(BOTAN_HAS_KYBER)
+   if(is_modern()) {
+      return true;
+   }
+#endif
+
+#if defined(BOTAN_HAS_KYBER_90S)
+   if(is_90s()) {
+      return true;
+   }
+#endif
+
+   return false;
+}
+
 namespace {
 
 class KyberConstants {
@@ -378,8 +416,8 @@ class Polynomial {
          for(size_t i = 0; i < size() / 8; ++i) {
             result[i] = 0;
             for(size_t j = 0; j < 8; ++j) {
-               const uint16_t t = (((static_cast<uint16_t>(this->m_coeffs[8 * i + j]) << 1) + KyberConstants::Q / 2) /
-                                   KyberConstants::Q);
+               const uint16_t t =
+                  ct_int_div_kyber_q((static_cast<uint16_t>(this->m_coeffs[8 * i + j]) << 1) + KyberConstants::Q / 2);
                result[i] |= (t & 1) << j;
             }
          }
@@ -531,7 +569,7 @@ class Polynomial {
        */
       static int16_t barrett_reduce(int16_t a) {
          int16_t t;
-         const int16_t v = ((1U << 26) + KyberConstants::Q / 2) / KyberConstants::Q;
+         constexpr int16_t v = ((1U << 26) + KyberConstants::Q / 2) / KyberConstants::Q;
 
          t = static_cast<int32_t>(v) * a >> 26;
          t *= KyberConstants::Q;
@@ -832,8 +870,7 @@ class Ciphertext {
             size_t offset = 0;
             for(size_t i = 0; i < p.size() / 8; ++i) {
                for(size_t j = 0; j < 8; ++j) {
-                  t[j] =
-                     (((static_cast<uint16_t>(p[8 * i + j]) << 4) + KyberConstants::Q / 2) / KyberConstants::Q) & 15;
+                  t[j] = ct_int_div_kyber_q((static_cast<uint16_t>(p[8 * i + j]) << 4) + KyberConstants::Q / 2) & 15;
                }
 
                r[0 + offset] = t[0] | (t[1] << 4);
@@ -846,8 +883,7 @@ class Ciphertext {
             size_t offset = 0;
             for(size_t i = 0; i < p.size() / 8; ++i) {
                for(size_t j = 0; j < 8; ++j) {
-                  t[j] =
-                     (((static_cast<uint32_t>(p[8 * i + j]) << 5) + KyberConstants::Q / 2) / KyberConstants::Q) & 31;
+                  t[j] = ct_int_div_kyber_q((static_cast<uint32_t>(p[8 * i + j]) << 5) + KyberConstants::Q / 2) & 31;
                }
 
                r[0 + offset] = (t[0] >> 0) | (t[1] << 5);
@@ -1149,9 +1185,9 @@ class Kyber_KEM_Decryptor final : public PK_Ops::KEM_Decryption_with_KDF,
 
          // Overwrite pre-k with z on re-encryption failure (constant time)
          secure_vector<uint8_t> lower_g_out_final(lower_g_out.size());
-         const uint8_t reencrypt_success =
-            constant_time_compare(encapsulated_key.data(), cmp.data(), encapsulated_key.size());
          BOTAN_ASSERT_NOMSG(lower_g_out.size() == m_key.m_private->z().size());
+
+         const auto reencrypt_success = CT::is_equal(encapsulated_key.data(), cmp.data(), encapsulated_key.size());
          CT::conditional_copy_mem(reencrypt_success,
                                   lower_g_out_final.data(),
                                   lower_g_out.data(),
@@ -1177,10 +1213,6 @@ KyberMode Kyber_PublicKey::mode() const {
    return m_public->mode().mode();
 }
 
-std::string Kyber_PublicKey::algo_name() const {
-   return object_identifier().to_formatted_string();
-}
-
 AlgorithmIdentifier Kyber_PublicKey::algorithm_identifier() const {
    return AlgorithmIdentifier(mode().object_identifier(), AlgorithmIdentifier::USE_EMPTY_PARAM);
 }
@@ -1239,6 +1271,10 @@ bool Kyber_PublicKey::check_key(RandomNumberGenerator&, bool) const {
    return true;  // ??
 }
 
+std::unique_ptr<Private_Key> Kyber_PublicKey::generate_another(RandomNumberGenerator& rng) const {
+   return std::make_unique<Kyber_PrivateKey>(rng, mode());
+}
+
 Kyber_PrivateKey::Kyber_PrivateKey(RandomNumberGenerator& rng, KyberMode m) {
    KyberConstants mode(m);
 
