@@ -601,6 +601,26 @@ std::vector<Test::Result> PK_Key_Generation_Test::run() {
          result.test_eq(
             "public_key has same encoding", Botan::X509::PEM_encode(key), Botan::X509::PEM_encode(*public_key));
 
+         // Test generation of another key pair from a given (abstract) asymmetric key
+         // KEX algorithms must support that (so that we can generate ephemeral keys in
+         // an abstract fashion). For other algorithms it's a nice-to-have.
+         try {
+            auto sk2 = public_key->generate_another(Test::rng());
+            auto pk2 = sk2->public_key();
+
+            result.test_eq("new private key has the same name", sk2->algo_name(), key.algo_name());
+            result.test_eq("new public key has the same name", pk2->algo_name(), public_key->algo_name());
+            result.test_eq(
+               "new private key has the same est. strength", sk2->estimated_strength(), key.estimated_strength());
+            result.test_eq("new public key has the same est. strength",
+                           pk2->estimated_strength(),
+                           public_key->estimated_strength());
+            result.test_ne("new private keys are different keys", sk2->private_key_bits(), key.private_key_bits());
+         } catch(const Botan::Not_Implemented&) {
+            result.confirm("KEX algorithms are required to implement 'generate_another'",
+                           !public_key->supports_operation(Botan::PublicKeyOperation::KeyAgreement));
+         }
+
          // Test PEM public key round trips OK
          try {
             Botan::DataSource_Memory data_src(Botan::X509::PEM_encode(key));
@@ -783,6 +803,13 @@ class PK_API_Sign_Test : public Text_Based_Test {
          result.confirm("public key claims to support signatures",
                         pubkey->supports_operation(Botan::PublicKeyOperation::Signature));
          result.test_gt("Public key length must be greater than 0", privkey->key_length(), 0);
+         if(privkey->stateful_operation()) {
+            result.confirm("A stateful key reports the number of remaining operations",
+                           privkey->remaining_operations().has_value());
+         } else {
+            result.confirm("A stateless key has an unlimited number of remaining operations",
+                           !privkey->remaining_operations().has_value());
+         }
 
          auto signer = std::make_unique<Botan::PK_Signer>(
             *privkey, Test::rng(), sig_params, Botan::Signature_Format::Standard, provider);
